package no.ntnu.idatt1002.demo.data.Budget;

import no.ntnu.idatt1002.demo.data.Economics.*;

import java.io.*;

/**
 * FileHandling is a class for writing and reading
 * Budgets to and from a file.
 *
 * @author andreas
 */
public class FileHandlingBudget {
    private static final String filePath = "src/main/resources/Budget/";
    private static final String fileType = ".budget";
    private static final String maxAmount = "maxAmount=";
    private static final String budgetAmount = "budgetAmount=";
    private static final String budgetCategory = "budgetCategory=";
    private static final String budgetDescription = "budgetDescription=";

    /**
     * Method for writing (adding) a budget to a file.
     *
     * @param generalBudget the budget you want to write to a file.
     * @param fileTitle the name of the file you want to check
     * @throws IOException if an input or output exception occurred.
     */
    public static void writeGeneralBudgetToFile(String fileTitle, GeneralBudget generalBudget) throws IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter(filePath + fileTitle + fileType))) {
            bw.write(generalBudget.toString());
        } catch (IOException ex) {
            throw new IOException("Error writing story to file: " + ex.getMessage());
        }
    }

    /**
     * Method for checking if a .budget file is empty.
     *
     * @param fileTitle the name of the file you want to check
     * @return true or false depending on if file is empty.
     * @throws IOException if an input or output exception occurred.
     */
    public static boolean isEmpty(String fileTitle) throws IOException {
        try (BufferedReader br = new BufferedReader(new FileReader(filePath + fileTitle + fileType))) {
            return br.readLine() == null;
        }
    }

    /**
     * Method for checking if a .budget file is new (no categories)
     * or old (has budget categories)
     * @param fileTitle The name of the file
     * @return True, if the budget is new. Else, returns false
     * @throws IOException
     */
    public boolean isNewBudget(String fileTitle) throws IOException {
        try (BufferedReader br = new BufferedReader(
            new FileReader(filePath + fileTitle + fileType))) {

            for (int i = 0; i < 3; ++i)
                br.readLine();

            if (br.readLine() == null) {
                return true;
            }
        }
        return false;
    }

    /**
     * Method for reading (getting) a Budget from a file.
     *
     * @param fileTitle the name of the file you want to read from.
     * @return the GeneralBudget from the file.
     * @throws IOException if an input or output exception occurred.
     */
    public static GeneralBudget readGeneralBudgetFromFile(String fileTitle) throws IOException {
        GeneralBudget generalBudget = null;
        double maxAmount = 0;
        double budgetAmount = 0;
        ExpenseCategory expenseCategory = null;
        String budgetDescription = null;

        try (BufferedReader br = new BufferedReader(new FileReader(filePath + fileTitle + fileType))) {
            String line;
            String nextLine = br.readLine();
            while ((line = nextLine) != null) {
                nextLine = br.readLine();
                if (line.startsWith(FileHandlingBudget.maxAmount)) {
                    maxAmount = Double.parseDouble(line.replace(FileHandlingBudget.maxAmount,""));
                } else if (line.startsWith(FileHandlingBudget.budgetAmount)) {
                    budgetAmount = Double.parseDouble(line.replace(FileHandlingBudget.budgetAmount,""));
                } else if (line.startsWith(FileHandlingBudget.budgetCategory)) {
                    line = line.replace(FileHandlingBudget.budgetCategory, "");
                    expenseCategory = switch (line) {
                        case "FOOD" -> ExpenseCategory.FOOD;
                        case "CLOTHES" -> ExpenseCategory.CLOTHES;
                        case "BOOKS" -> ExpenseCategory.BOOKS;
                        default -> ExpenseCategory.OTHER;
                    };
                } else if (line.startsWith(FileHandlingBudget.budgetDescription)) {
                    budgetDescription = line.replace(FileHandlingBudget.budgetDescription,"");
                }
                if (line.isEmpty() || (nextLine == null)) {
                    if(generalBudget == null){
                        generalBudget = new GeneralBudget(maxAmount);
                    } else{
                        generalBudget.addToBudget(budgetAmount,budgetDescription,expenseCategory);
                    }
                }
            }
        }
        return generalBudget;
    }
}
